@@ -3,15 +3,14 @@ require_relative "./lazylib.rb"
 
 SepRx = /, | |,/
 NumRx = /-?\d+/
-BetweenRx = /#{SepRx}|#{NewlineRx}|#{SepRx}#{NewlineRx}/
 
 def parse_input
-  input = to_eager_str($ReadStdin.value)
-
-  lazy_input = lines($ReadStdin)
+  lazy_input = $ReadStdinLines
+  eager_input = to_eager_list(lazy_input)
   # is it just numbers?
-  if input =~ /\A#{NumRx}(#{BetweenRx}#{NumRx})*#{BetweenRx}?\Z/m
-    read = map(lazy_input.const){|v|split_non_digits(v)}
+  if eager_input.all?{|line| to_eager_str(line) =~ /^(#{NumRx}#{SepRx})*#{NumRx}?$/ } &&
+     eager_input.any?{|line| to_eager_str(line) =~ NumRx }
+    read = map(lazy_input){|v|split_non_digits(v)}
     eager = to_eager_list(read.const).map{|a| to_eager_list(a.const) }
 
     if eager.size == 1 && eager[0].size == 1 # is it 1 number
@@ -24,10 +23,10 @@ def parse_input
       [read, IntType, 2]
     end
   else # not numbers
-    if input =~ /#{NewlineRx}./m # multi line
-      [lazy_input, CharType, 2]
+    if eager_input.size > 1 # multi line
+      [lazy_input.value, CharType, 2]
     else
-      [lazy_input.empty? ? [] : lazy_input[0].value, CharType, 1, lazy_input.empty?]
+      [lazy_input.empty ? [] : lazy_input.value[0].value, CharType, 1, lazy_input.empty]
     end
   end
 end
\ No newline at end of file
