@@ -1,14 +1,14 @@
 IR = Struct.new(:op, :args, :token)
 
-def to_ir(ast_nodes, arg, registers, override_input)
+def to_ir(ast_nodes, arg, registers, override_input, ast_inds)
   nodes = [IR.new(arg, [], nil)]
-  ast2ir = { 0 => 0 }
+  ast2ir = { 0 => override_input ? ->{ast2ir[registers[RegisterName]]} : 0 }
   ast_nodes.size.times{|ast_ind| to_ir_h(ast_nodes, ast_ind, nodes, ast2ir, registers) }
   replace_vars = lambda{|a| a=a[] while Proc === a; a }
   nodes.each{|n|n.args.map! &replace_vars }
   ast2ir.each{|k,v| ast2ir[k] = replace_vars[v] }
-  nodes[0].args[0] = ast2ir[registers[RegisterName]] if override_input
-  [nodes, ast2ir]
+  ir_inds = ast_inds.map{|i| ast2ir[i] }
+  [nodes, ir_inds]
 end
 
 def to_ir_h(ast_nodes, ast_ind, nodes, ast2ir, registers)
@@ -52,11 +52,10 @@ def to_ir_h(ast_nodes, ast_ind, nodes, ast2ir, registers)
       elsif op.name == "iterate0"
         nodes << IR.new(mod > 0 ? Ops["zerom"].dup_mod(mod) : Ops["zero"], zero_args=[nil], ast_node.token)
         nodes << IR.new(Ops["cons"].dup_mod(mod), cons_args=[nil, nodes.size-1], ast_node.token)
-        consed_ind = nodes.size - 1
         nodes << IR.new(Ops["superpad"].dup_mod(mod), [nodes.size-1], ast_node.token)
         ast2ir[ast_node.block_arg] = nodes.size - 1
         cons_args[0] = zero_args[0] = to_ir_h(ast_nodes, ast_node.block, nodes, ast2ir, registers)
-        consed_ind
+        cons_args[0]
       elsif op.name == "foldr"
         nodes << IR.new(Ops["superpad"].dup_mod(mod), superpad_args=[nil], ast_node.token)
         nodes << IR.new(Ops["pad"].dup_mod(mod), [nodes.size-1, args[0]], ast_node.token)
