@@ -1,18 +1,22 @@
-require_relative "./promise.rb"
+require "set"
 
-def output(str)
-   print(str)
-end
+def make_promises(ir2)
+  promises = ir2.map{|node|
+    arg_types = node.args.map{|a|ir2[a].type}
 
-def error_output(str)
-   $stderr.write(str)
-end
+    impl = lambda{|*args|
+      $rank = node.way.vectorize ? node.way.mod : ir2[node.args[0]].rank
+      $at = arg_types[0]
+      $bt = arg_types[1]
+      node.way.impl[*args]
+    }
 
-def zipn(n, a, f)
-  return f[*a] if n <= 0 || a==[]
-  return [] if a.any?{|i|i.empty}
-  [promise{zipn(n-1, a.map{|i|i.value[0]}, f) },
-   promise{zipn(n, a.map{|i|i.value[1]}, f) }]
+    promise{
+      args = node.args.map{|a| promises[a] }
+      zipn(node.zip_level, args, impl)
+    }
+  }
+  promises
 end
 
 class Promise
@@ -37,22 +41,25 @@ class Object
   end
 end
 
+def zipn(n, a, f)
+  return f[*a] if n <= 0 || a==[]
+  return [] if a.any?{|i|i.empty}
+  [promise{zipn(n-1, a.map{|i|i.value[0]}, f) },
+   promise{zipn(n, a.map{|i|i.value[1]}, f) }]
+end
+
+def output(str)
+   print(str)
+end
+
+def error_output(str)
+   $stderr.write(str)
+end
+
 def repeat(a)
   (ret = [a]) << ret.const
 end
 
-def repn(n, a)
-  n<=0 ? a : promise{ repeat(repn(n-1, a)) }
-end
-
-def promoten(n,a)
-  if n<=0
-    a
-  else
-    [promoten(n-1,a),Null].const
-  end
-end
-
 def concat_map(v,rhs=Null,first=true,&b)
   if v.empty
     rhs.value
@@ -154,52 +161,11 @@ def pow(a,b)
   neg_exponent ? 1 / y : y
 end
 
-def printv(a, type, rank)
-  srank = rank + (type == IntType ? 1 : 0)
-  n = [srank == 2 ? 1 : srank - 2, 0].max
-  printn(a, n, type, srank - n)
-end
-
-def printn(a, n, type, srank)
-  if n == 0
-    if srank == 0
-      prints([a.const, Null])
-    elsif srank == 1
-      case type
-      when EmptyType
-
-      when IntType
-        output a.to_s
-      when CharType
-        prints a
-      else raise
-      end
-    elsif srank == 2
-      first = true
-      until a.empty?
-        output " " if !first
-        printn(a[0].value, 0, type, 1)
-        a = a[1].value
-        first = false
-      end
-    else
-      raise
-    end
-  else
-    until a.empty?
-      printn(a[0].value, n-1, type, srank)
-      output("\n")
-      a = a[1].value
-    end
-  end
-end
-
 def prints(value)
   while !value.empty?
     v = value[0].value
     # raw print, multiple bytes will show up as one if utf is used. output will match source code so will look correct if editor encoding=terminal encoding
-    raise IogiiError.new("char value, %s, outside of byte range" % v, nil) if v<0 || v>255
-    output v.chr
+    output((v % 256).chr)
     value = value[1].value
   end
 end
@@ -433,15 +399,6 @@ def chunk_while(a,b,btype,rank)
    promise{ truthy.value ? rest.value[1].value : rest.value }]
 end
 
-def match(a,b)
-  haystack = to_eager_str(a.value)
-  regex = to_eager_str(b.value)
-
-  results = []
-  haystack.scan(Regexp.new(regex)){results << str_to_lazy_list($&) }
-  to_lazy_list(results)
-end
-
 def strip(a,at,rank)
   lstrip = drop_until(a,a,at,rank)
   b = reverse(lstrip.const).const
@@ -500,15 +457,18 @@ def group_while(a,b,rank,btype)
   ]
 end
 
-def reshape(a,b)
-  return [] if b.empty || a.empty && b.value[0].value>0
-  [Promise.new{ take(b.value[0].value,a) },
-   Promise.new{ reshape(Promise.new{ drop(b.value[0].value,a) }, b.value[1]) }]
-end
-
-# truncate as soon as encounter empty list
-def trunc(a)
-  a.empty || a.value[0].empty ? [] : [a.value[0], Promise.new{trunc(a.value[1])}]
+def reshape(a, b)
+  return [] if b.empty
+  n = b.value[0].value
+  return [] if n>0 && a.empty
+  return [Null, promise{ reshape(a, b.value[1]) }] if n <= 0
+  [promise{ take(n, a) },
+   promise{
+    d = drop(n-1, a)
+    # took more than all, next is []
+    d == [] ? [] : reshape(d[1], b.value[1])
+   }
+  ]
 end
 
 def map(a,&b)
@@ -525,14 +485,21 @@ def concat(a)
   concat_map(a){|i,r,first|append(i,r)}
 end
 
-def transpose(a)
+def rstrip(a)
   return [] if a.empty
-  return transpose(a.value[1]) if a.value[0].empty
-  broken = trunc(a.value[1]).const
-  hds = Promise.new{ map(broken){|v|v.value[0].value} }
-  tls = Promise.new{ map(broken){|v|v.value[1].value} }
-  [Promise.new{[a.value[0].value[0],hds]},
-   Promise.new{transpose [a.value[0].value[1],tls].const}]
+  rest = promise{ rstrip(a.value[1]) }
+  return [] if a.value[0].empty && rest.empty
+  [a.value[0], rest]
+end
+
+def inf_tails(a)
+  ans=[a, promise{ map(ans.const){|e| map(e){|e2| e2.empty ? [] : e2.value[1].value }} }]
+end
+
+def transpose(a,at,rank)
+  z = promise{ map(promise{ inf_tails(a) }){|e| rstrip(e) } }
+  t = promise{ map(z){|e| map(e){|e2| e2.empty ? zero(rank,at) : e2.value[0].value } } }
+  take_while(t,t,at,rank+1)
 end
 
 def square_root(x,n)
@@ -545,6 +512,17 @@ def square_root(x,n)
   guess
 end
 
+CharClasses = [
+  [*'a'..'z'],
+  [*'A'..'Z'],
+  [*'0'..'9'],
+  [*' '..126.chr] - ([*'a'..'z'] + [*'A'..'Z'] + [*'0'..'9'] + [' ',"\n"]),
+  [' ',"\n"]
+].map{|chars| chars.map(&:ord).to_set }
+def char_class(a,b)
+  CharClasses.each.with_index.any?{|clazz, i| b[i] == 1 && clazz.include?(a) } ? 1 : 0
+end
+
 def to_base(a,b)
   raise IogiiError.new "base 0", nil if b==0
   reverse(to_base_h(a.abs,b,a<=>0).const)
@@ -572,7 +550,13 @@ def undigits(a)
   }.const), 10)
 end
 
-$ReadStdin = Promise.new{ read_stdin }
+$ReadStdinLines = Promise.new{
+  if $raw_args
+    to_lazy_list($raw_args.map{|a| str_to_lazy_list(a) })
+  else
+    lines(Promise.new{ read_stdin })
+  end
+}
 def read_stdin
   c=STDIN.getc
   if c.nil?
