@@ -14,7 +14,7 @@ class Op < Struct.new(:name, :parse_nargs, :block, :ways)
   end
 end
 
-class Way < Struct.new(:arg_ranks, :ret_rank, :arg_types, :ret_type, :mod, :vectorize, :promote, :impl)
+class Way < Struct.new(:name, :arg_ranks, :ret_rank, :arg_types, :ret_type, :mod, :vectorize, :promote, :impl)
   def mod_arg_ranks
     arg_ranks.map{|r| r < 0 ? ~r + mod : r }
   end
@@ -22,7 +22,7 @@ class Way < Struct.new(:arg_ranks, :ret_rank, :arg_types, :ret_type, :mod, :vect
     ret_rank < 0 ? ~ret_rank + mod : ret_rank
   end
   def dup
-    Way.new(arg_ranks.dup, ret_rank, arg_types.dup, ret_type, mod, vectorize, promote, impl)
+    Way.new(name, arg_ranks.dup, ret_rank, arg_types.dup, ret_type, mod, vectorize, promote, impl)
   end
   def can_mod?
     (arg_ranks + [ret_rank]).any?{|r| r < 0 }
@@ -45,7 +45,7 @@ def new_op(name, type_sig, block: false, vectorize: true, promote: true, &impl)
   *arg_ranks, ret_rank = parts.map{|arg| spec_rank(arg) }
   *arg_types, ret_type = parts.map{|arg| spec_type(arg) }
 
-  way = Way.new(arg_ranks, ret_rank, arg_types, ret_type, 0, vectorize, promote, impl)
+  way = Way.new(name, arg_ranks, ret_rank, arg_types, ret_type, 0, vectorize, promote, impl)
   Op.new(name, arg_ranks.size, block, [way])
 end
 
@@ -55,7 +55,7 @@ OpExamples = {}
 if !defined? mk_op
   def mk_op(names, type_sig, example="", block: false, vectorize: true, promote: true, &impl)
     OpExamples[names] = example if example != ""
-    names = names[/.[^(]*/].split
+    names = names.split
 
     op = new_op(names[-1], type_sig, block: block, vectorize: vectorize, promote: promote, &impl)
 
@@ -95,7 +95,7 @@ RepSnd = :rep_second
 
 category("mathy ops")
 mk_op("+ add", "int x -> x", '1 2+ -> 3', &add=->a,b{ a.value+b.value })
-mk_op("+ add", "char int -> char", "'a 1+ -> 'b", &add)
+mk_op("+ add", "char int -> char", "'a 1+ -> 'b'", &add)
 mk_op("- sub", "x int -> x", '5 3- -> 2', &subt=->a,b{ a.value-b.value })
 mk_op("- sub", "char char -> int", "'b 'a- -> 1", &subt)
 mk_op("* mult", "int int -> int", '2 3* -> 6'){|a, b| a.value*b.value }
@@ -105,23 +105,23 @@ mk_op("^ pow", "int int -> int", '2 3^ -> 8'){|a, b| pow(a.value, b.value) }
 mk_op("_ negate", "int -> int", '1_ -> -1'){|a| -a.value }
 mk_op("| abs", "int -> int", '5~| -> 5'){|a| a.value.abs }
 mk_op("( pred", "x -> x", '4( -> 3'){|a| a.value-1 }
-mk_op(") succ", "x -> x", "'a) -> 'b"){|a| a.value+1 }
+mk_op(") succ", "x -> x", "'a) -> 'b'"){|a| a.value+1 }
 mk_op("} countTo", "int -> [int]", "3} -> [1,2,3]"){|a| range(1,a.value+1) }
 mk_op("{ wholes", "[int]", "{ -> [0,1,2,..."){ range_from(0) }
 mk_op("# sum", "[x] -> int", '1,2,3,4 # -> 10', promote: false){|a| to_eager_list(a).sum }
 
 category("char ops")
-mk_op("+ match", "[char] [char] -> [[char]]", '"a12b3" "\d+"+ -> ["12","3"]'){|a,b| match(a,b) }
 mk_op("^ charRange", "char char -> [char]", "'a'f^ -> \"abcdef\""){|a,b| range(a.value, b.value+1) }
 mk_op("_ read", "[char] -> int", '"23"_ -> 23'){|a| read_num(a)[0] }
 mk_op("} readAll", "[char] -> [int]", '"1,2 3"} -> [1,2,3]'){|a| split_non_digits(a) }
 mk_op("| ord", "char -> int", "'d| -> 100"){|a| a.value }
-mk_op("|, chr", "int -> char", "100|, -> 'd"){|a| a.value }
+mk_op("|, chr", "int -> char", "100|, -> 'd'"){|a| a.value }
 mk_op("` str", "int -> [char]", '5` -> "5"'){|a| str(a) }
-mk_op("` strip", "[char] -> [char]", '" ab\n"` -> "ab"'){|a| strip(a,$at,$rank) }
+mk_op("` strip", "[char] -> [char]", '" a b\n"` -> "a b"'){|a| strip(a,$at,$rank) }
 mk_op("* replicate", "[char] int -> [char]", '"ab"3* -> "ababab"'){|a, b|
   concat(take(b.value,repeat(a).const).const) }
 mk_op("* join", "[[char]] [[char]] -> [char]", '1,2,3 " "* -> "1 2 3"', promote: RepSnd){|a, b| join(a,b)}
+mk_op("/ luds charClass", "char int -> int", '"Ab$"9/ -> [0,1,1]'){|a,b| char_class(a.value, b.value) }
 mk_op("/ split", "[char] [char] -> [[char]]", '"a  b c " " "/ -> ["a","b","c"]'){|a, b| s=cut(a,repeat(b).const).const; filter(s,s,nil,1) }
 mk_op("% cut splitKeepEmpties", "[char] [[char]] -> [[char]]",'"a  b c " " "% -> ["a","","b","c",""]', promote: RepSnd){|a, b| cut(a,b) }
 
@@ -130,15 +130,15 @@ mk_op("a append", "[a] [a] -> [a]", '"ab" "cd"a -> "abcd"'){|a, b| append(a,b) }
 mk_op("b backwards reverse", "[a] -> [a]", '"abc"b -> "cba"', promote: false){|a| reverse(a) }
 mk_op("c cons", "[a] a -> [a]", '1,2 0c -> [0,1,2]'){|a, b| [b, a] }
 mk_op("d drop", "[a] int -> [a]", '"abcde" 2d -> "cde"'){|a, b| drop(b.value, a) }
-mk_op("g get", "[a] int -> a", '"abcd" 2g -> \'c'){|a, b| z=zero($rank,$at); a = drop(b.value, a); a==[] ? z : a[0].value }
-mk_op("h head", "[a] -> a", '"abc"h -> \'a', promote: false){|a| z=zero($rank,$at); a.empty ? z : a.value[0].value }
+mk_op("g get", "[a] int -> a", '"abcd" 2g -> \'c\''){|a, b| z=zero($rank,$at); a = drop(b.value, a); a==[] ? z : a[0].value }
+mk_op("h head", "[a] -> a", '"abc"h -> \'a\'', promote: false){|a| z=zero($rank,$at); a.empty ? z : a.value[0].value }
 mk_op("j just", "a -> [a]", "1j -> [1]"){|a|[a, [].const]}
 mk_op("k keep take", "[a] int -> [a]", '"abcde" 2k -> "ab"'){|a, b| take(b.value, a) }
-mk_op("l last", "[a] -> a", '"abc"l -> \'c', promote: false){|a| z=zero($rank,$at); last(a, z.const) }
+mk_op("l last", "[a] -> a", '"abc"l -> \'c\'', promote: false){|a| z=zero($rank,$at); last(a, z.const) }
 mk_op("n not", "a -> int", '0,1,2n -> [1,0,0]', promote: false){|a| truthy($at, $rank, a.value) ? 0 : 1 }
-mk_op("O Or o, or,", "int [char] -> [char]", promote: NotFirst){|a,b| truthy($at, 0, a.value) ? str(a) : b.value }
+mk_op("O Or o, or, or2", "int [char] -> [char]", promote: NotFirst){|a,b| truthy($at, 0, a.value) ? str(a) : b.value }
 mk_op("o or", "a a -> a", '" bc" \'- o -> "-bc"', promote: NotFirst){|a,b| truthy($at, $rank, a.value) ? a.value : b.value }
-mk_op("p pivot transpose", "[[a]] -> [[a]]", '"abc","def"p -> ["ad","be","cf"]', promote: false){|a| transpose(a) }
+mk_op("p pivot transpose", "[[a]] -> [[a]]", '"abc","def"p -> ["ad","be","cf"]', promote: false){|a| transpose(a, $at, $rank) }
 mk_op("q quantity len", "[a] -> int", '"abc"q -> 3', promote: false){|a| len(a.value) }
 mk_op("r repeat", "a -> [a]", "1r -> [1,1,1,1,1,1..."){|a| repeat(a) }
 mk_op("s sortBy", "[a] [b] -> [a]", '"abc","d":len S -> ["d","abc"]', promote: false){|a,b| sort_by(a,b,$rank) }
@@ -159,7 +159,7 @@ mk_op("\\ chunkWhen", "[a] [b] -> [[a]]", '"ab cd e":\\ -> ["ab ","cd ","e"]', p
 
 category("folding ops") # (return type is type yielded)
 mk_op("i iterate", "a -> [a]", '0 i 1,2,3+ -> [0,1,3,6]', block: true)
-mk_op("e expand iterate0", "[a]", 'e 1,2,3+ -> [0,1,3,6]', block: true)
+mk_op("e expand iterate0", "[a]", 'e 1,2,3+ -> [1,3,6]', block: true)
 mk_op("f foldr", "a -> [a]", '0 f 1,2,3+ -> 6', block: true)
 mk_op("m meld foldr0", "[a]", 'm 1,2,3+ -> 6', block: true)
 
@@ -176,14 +176,15 @@ quickref_op("@", "register get/set", "@( 5@) -> 4 6")
 quickref_op("[ ]", "push pop", "5[( ]) -> 4 6")
 quickref_op("~", "assignment", "5~a( a) -> 4 6")
 
+# available for char +
 category("low rank overloads")
 # Available (for int):
 # GSV  2 arg
 # LZ   1 arg
-# IF   block using
+# FI   block using
 
-mk_op("U uppercase", "char -> char", "'a U -> 'A"){|a| a=a.value; a>='a'.ord && a<='z'.ord ? a-32 : a }
-mk_op("L lowercase", "char -> char", "'A L -> 'a"){|a| a=a.value; a>='A'.ord && a<='Z'.ord ? a+32 : a }
+mk_op("U uppercase", "char -> char", "'a U -> 'A'"){|a| a=a.value; a>='a'.ord && a<='z'.ord ? a-32 : a }
+mk_op("L lowercase", "char -> char", "'A L -> 'a'"){|a| a=a.value; a>='A'.ord && a<='Z'.ord ? a+32 : a }
 mk_op("P parts splitWS", "[char] -> [[char]]", '"ab c\n d" P -> ["ab","c","d"]'){|a| x=promise{ group_while(a,a,0,CharType) }; filter(x,x,CharType,1) }
 
 mk_op("p digits", "int -> [int]", '123 p -> [1,2,3]'){|a| v=to_base(a.value,10); v.empty? ? [0.const, Null] : v }
@@ -205,11 +206,10 @@ mk_op("pad", "[a] a -> [a]", '1,2 3 pad -> [1,2,3,3,3...'){|a, b| padv(a, b) }
 mk_op("show", "a -> [char]", '1,2 show -> "[1,2]"', vectorize: false, &show=->a{ inspectv($at, $rank, a) })
 mk_op("type", "a -> [char]", '1,2 type -> "[int]"', vectorize: false){|a| str_to_lazy_list(type_and_rank_to_str($at, $rank)) }
 mk_op("version", "[char]", "version -> \"#{Version}, ruby#{RUBY_VERSION}\""){ str_to_lazy_list(Version + ", ruby#{RUBY_VERSION}") }
-mk_op("pp", "a -> [char]", '1,2 => prints [1,2]', vectorize: false, &show)
-mk_op("nuke", "a -> a", '1 2 nuke -> 1', vectorize: false){|a| raise}
+mk_op("del", "a -> a", '1 2 del -> 1', vectorize: false){|a| raise}
 
 category("internal ops")
-mk_op("lines", "[char] -> [[char]]"){|a| lines(a) }
+# mk_op("lines", "[char] -> [[char]]"){|a| lines(a) }
 mk_op("superpad", "[a] -> [a]", promote: false){ raise }
 mk_op("zero", "[] -> int"){|a| 0 } # default to int in fold0s
 mk_op("zerom", "[a] -> a"){|a| zero($rank,$at) } # this is used for rank1+ fold0s
@@ -217,9 +217,16 @@ mk_op("zero", "[a] -> a"){|a| zero($rank,$at) }
 
 # todo do this without duplicating logic/names
 category("intuitive special promotes (don't need to show them in quickref)")
-mk_op("* join", "[char] [[char]] -> [char]", promote: [RepSnd,NotFirst]){|a, b| join(map(a){|i| [i,Null] }.const,b)} # ideally don't promote first at all
-mk_op("S SortBy sortBy, s,", "[[a]] [b] -> [[a]]", promote: false){|a,b| sort_by(a,b,$rank) }
-mk_op("V Vet Filter vet, filter, v,", "[[a]] [b] -> [[a]]", promote: false){|a,b| filter(a,b,$bt,$rank) }
-mk_op("W While TakeWhile w, while, takeWhile,", "[[a]] [b] -> [[a]]", promote: false){|a,b| take_while(a,b,$bt,$rank-1) }
-mk_op("\\, ChunkWhen chunkWhen,", "[[a]] [b] -> [[[a]]]", promote: false){|a,b| chunk_while(a,b,$bt,$rank) }
-mk_op("Y y, IfElse ifElse,", "[a] b b -> b", promote: NotFirst){|a,b,c| truthy($at, $rank+1, a.value) ? b.value : c.value }
+mk_op("* join join2", "[char] [[char]] -> [char]", promote: [RepSnd,NotFirst]){|a, b| join(map(a){|i| [i,Null] }.const,b)} # ideally don't promote first at all
+mk_op("S SortBy sortBy, s, sortBy2", "[[a]] [b] -> [[a]]", promote: false){|a,b| sort_by(a,b,$rank) }
+mk_op("V Vet Filter vet, filter, v, filter2", "[[a]] [b] -> [[a]]", promote: false){|a,b| filter(a,b,$bt,$rank) }
+mk_op("W While TakeWhile w, while, takeWhile, takeWhile2", "[[a]] [b] -> [[a]]", promote: false){|a,b| take_while(a,b,$bt,$rank-1) }
+mk_op("\\, ChunkWhen chunkWhen, chunkWhen2", "[[a]] [b] -> [[[a]]]", promote: false){|a,b| chunk_while(a,b,$bt,$rank) }
+mk_op("Y y, IfElse ifElse, ifElse2", "[a] b b -> b", promote: NotFirst){|a,b,c| truthy($at, $rank+1, a.value) ? b.value : c.value }
+
+Unlines = Way.new("unlines", [2], [1], [CharType], CharType, 0, nil, nil, ->a{
+  concat(map(a){|e| append(e, [10.const, Null].const) }.const)
+})
+Unwords = Way.new("unwords", [2], [1], [CharType], CharType, 0, nil, nil, ->a{
+  join(a, repeat([32.const, Null].const).const)
+})
\ No newline at end of file
