@@ -5,34 +5,25 @@ AST = Struct.new(:token, :args)
 BlockAST = Struct.new(:token, :args, :block, :block_arg)
 ArgNode = Struct.new(:arg_ind)
 VarNode = Struct.new(:name)
-DataNode = Struct.new(:impl)
+DataNode = Struct.new(:impl, :str)
 
 RegisterName = "@ register"
 
 class Stack
-  def initialize(arg_optional, arg_ind)
-    @arg_ind = arg_ind
-    @stack = [arg_ind]
-    @arg_needed = !arg_optional
+  def initialize
+    @stack = []
     @negative_used = 0
   end
-  def initialize_copy(from)
-    @stack = from.stack
-    @arg_needed = from.arg_needed
-    @negative_used = negative_used
-  end
   def push(ind)
     @stack << ind
     self
   end
-  attr_reader :stack, :arg_needed, :negative_used
   def pop
       # return node inds lazily since negative stack size means future values that don't exist yet
     if @stack.empty?
       stack_ind = @negative_used += 1
       ->{@stack[-stack_ind]} # only valid once stack size is finalized
     else
-      @arg_needed |= @stack.size == 1
       @stack.pop
     end
   end
@@ -44,10 +35,9 @@ class Stack
   def size
     @stack.size - @negative_used
   end
-  def finalize(can_have_empty)
-    implicit_args = false
-    ((can_have_empty ? 0 : 1)-size).times{ implicit_args = true; @stack << @arg_ind }
-    @stack.shift if !@arg_needed && size > 1
+  def finalize(is_main, arg_ind)
+    implicit_args = is_main && @negative_used == 0 && size >= 1 ? 0 : [1-size, 1].max
+    implicit_args.times{ @stack << arg_ind }
     [@stack[0,size], implicit_args]
   end
 end
@@ -57,8 +47,7 @@ def parse_main(tokens)
   tokens = split_ids(tokens, registers)
   add_implicit_ends(tokens, registers)
   nodes = []
-  preprints = []
-  main_inds, _, block_arg_ind, token_ind=*parse(tokens, 0, nodes, true, nil, [], registers, false, preprints)
+  main_inds, _, block_arg_ind, token_ind=*parse(tokens, 0, nodes, true, nil, [], registers, false)
   raise if block_arg_ind != 0 # ir assumes 0 => 0
   raise IogiiError.new "program has an extra >", tokens[token_ind-1] if token_ind<=tokens.size
   nodes.each{|node|
@@ -74,9 +63,8 @@ def parse_main(tokens)
     end
   }
   main_inds.map!{|a| a=a.call while Proc===a; a}
-  preprints.map!{|a| a=a.call while Proc===a; a}
   registers.each{|k,v| registers[k] = v = v.call while Proc===v }
-  [nodes, main_inds, registers, preprints]
+  [nodes, main_inds, registers]
 end
 
 def split_ids(tokens, registers)
@@ -138,10 +126,11 @@ def add_implicit_ends(tokens, registers)
     begin_token = nil
     loop {
       begin_token = tokens[begins.last]
-      block_inds, implicit_args, _, _, could_end_earlier = *parse(tokens, begins.last + 1, [], false, DUMMY_STACK, DUMMY_STACK, registers, false, [])
+      block_inds, implicit_args, _, _, could_end_earlier = *parse(tokens, begins.last + 1, [], false, DUMMY_STACK, DUMMY_STACK, registers, false)
       begins.pop
       break if !could_end_earlier
-      break if block_inds.size == 1 && !implicit_args
+      # this block terminates normally
+      break if block_inds.size == 1 && implicit_args == 1
       # todo definitely not efficient to call this over and over
       break if !can_brackets_match(begins + all_begins.filter{|bi| bi >= e }, ends[ei..-1])
     }
@@ -150,9 +139,9 @@ def add_implicit_ends(tokens, registers)
   }
 end
 
-def parse(tokens, token_ind, nodes, is_main, outer, var_stack, registers, implicit_end, preprints)
+def parse(tokens, token_ind, nodes, is_main, outer, var_stack, registers, implicit_end)
   could_end_earlier = false
-  stack=Stack.new(is_main, nodes.size)
+  stack=Stack.new
   nodes << ArgNode.new(arg_ind = nodes.size)
 
   loop {
@@ -187,15 +176,16 @@ def parse(tokens, token_ind, nodes, is_main, outer, var_stack, registers, implic
       end
     else case t.type
     when :int, :char, :str
-      data_impl, token_ind = *parse_data(tokens, token_ind)
+      data_impl, token_ind = *parse_data(tokens, orig_token_ind=token_ind)
       stack.push nodes.size
-      nodes << DataNode.new(data_impl)
+      str = tokens[orig_token_ind-1...token_ind].map(&:str).join
+      nodes << DataNode.new(data_impl, str)
     when :id, :sym
       op = lookup_op(t.str)
       raise IogiiError.new "unknown op", t if !op
       args = op.parse_nargs.times.map{ stack.pop }.reverse
       if op.block
-        block_inds, _, block_arg_ind, token_ind, _ =*parse(tokens, token_ind, nodes, false, stack, var_stack, registers, !t.explicit_end, preprints)
+        block_inds, _, block_arg_ind, token_ind, _ =*parse(tokens, token_ind, nodes, false, stack, var_stack, registers, !t.explicit_end)
         raise IogiiError.new "excessive values in block and implicit ops not yet implemented", t if block_inds.size > 1
         raise if block_inds.size < 1
         block_ind = block_inds[0]
@@ -205,21 +195,19 @@ def parse(tokens, token_ind, nodes, is_main, outer, var_stack, registers, implic
       end
       stack.push nodes.size-1
       stack.push args[0] if op.name == "mdup" || op.name == "dup"
-      if "pp" == t.str
-        preprints << stack.pop
-      elsif "nuke" == t.str
+      if "del" == t.str
         stack.pop
       end
     when :commas
       raise IogiiError.new "commas must follow data or op that can vectorize", t
     else raise "unknown token type %p" % t.type.to_s
     end end
-    if stack.size == 1
+    if stack.size == 0
       break if implicit_end
       could_end_earlier = true
     end
   }
-  [*stack.finalize(is_main && preprints.size > 0), arg_ind, token_ind, could_end_earlier]
+  [*stack.finalize(is_main, arg_ind), arg_ind, token_ind, could_end_earlier]
 end
 
 def parse_data(tokens, token_ind)
