@@ -6,11 +6,13 @@ def output(str)
   $output << str
 end
 
-Examples.each{|name, prog, input, expected|
-  $ReadStdin = Promise.new{ str_to_lazy_list(input) }
+pass = true
+
+ExamplesAndTests.each{|name, prog, input, expected|
+  $ReadStdinLines = lines(Promise.new{ str_to_lazy_list(input) }).const
   begin
     $output = ""
-    run_and_do_io(prog)
+    run_and_do_io(prog,false,false)
 
     if $output != expected
       STDERR.puts "FAIL: #{name}"
@@ -19,11 +21,13 @@ Examples.each{|name, prog, input, expected|
       STDERR.puts expected
       STDERR.puts "found:"
       STDERR.puts $output
+      pass = false
     end
   rescue => e
+    pass = false
     STDERR.puts "error in "+name
     raise e
   end
 }
 
-puts "PASS #{Examples.size} examples"
+puts "PASS #{ExamplesAndTests.size} examples" if pass
