@@ -32,6 +32,7 @@ def mk_op(names, type_sig, example="", block: false, vectorize: true, promote: t
   if names == $last_names
     puts "".td
   else
+    names = (names.split - ["W,"])*" " # more of an implementation detail, hide ite
     puts names.tdcode(true)
     $last_names = names
   end
@@ -56,7 +57,7 @@ def mk_op(names, type_sig, example="", block: false, vectorize: true, promote: t
   end
 
   puts type_sig.tdcode
-  puts example.sub('+ (unstable)','').tdcode
+  puts example.sub(' (unstable)','').tdcode
   puts "</tr>"
 end
 
